package cologne.eck.all_peas.gui;

import java.awt.Window;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.io.File;
import java.util.ArrayList;

import javax.swing.JFrame;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.util.Arrays;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.CloudFilePanel;
import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.menu.SettingMenu;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Converter;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.Zeroizer;


@SuppressWarnings("serial")
public abstract class PeaLockFrame extends JFrame {
	
	private boolean changesApplied = true;
	
	/**
	 * Ask for further setting and if not:
	 * Change the password for current used files 
	 * or apply changes in algorithms and parameters:
	 * Ask for a new password, ask for other (remaining) files, 
	 * derive the key, encrypt the files
	 * 
	 * @param owner: PswDialogView or the lock frame
	 * @param fileNamesToChange	the file names to apply changes or null
	 * 							(apply to files from getCurrentFileNames() )
	 * @param algorithmChange	true: ask to make further settings,
	 * 							false for password change only
	 * 
	 * @return	null for success, a localized error message to show otherwise
	 */
	public String changePasswordOrApplyChanges(Window owner, String[] fileNamesToApply, boolean algorithmChange) {
		
		setChangesApplied(false);
		if ( ! PeaProperties.getFileType().equals("image")) { // TODO Notebook
			if (algorithmChange == true) {
				String[] options = { PeaProperties.getVmBridge().translate("apply_changes"), //"Apply changes", "make further settings"};
						PeaProperties.getVmBridge().translate("make_further_settings")};

				int answer = PeaDialog.showOptions(owner, 
						PeaProperties.getVmBridge().translate("apply_changes")+"?", PeaProperties.getVmBridge().translate("finalize_setting") + "?", 
						0, 3, options, options[1]);
				if (answer == -1) {// closed
					TestLog.o(getClass(), "Setting cancelled...");
					return null;
				} else if (answer == 1) { // make further settings
					TestLog.o(getClass(), "Make further settings...");
					//SettingMenu.setAlgoParamsBeforeChange(previousAlgoParams);
					return null;			
				} else if (answer == 0) { // apply

					// go on..
				}
			}
		}
		//String oldSaltAsHex = Converter.bytes2hex(KeyDerivation.getSalt());
		
		if (owner instanceof PswDialogView) {
			// although already done in SettingMenu and KDFSettings...
			String newDefaultString = AlgoParamHandler.writeString(true);
			PeaProperties.setDefaultAlgoParamString(newDefaultString);
			return null;

		} else if (owner instanceof PeaLockFrame) {
			String currentAlgos = AlgoParamHandler.writeString(false);
			PeaProperties.setAlgoParamString(currentAlgos.getBytes(AttachmentHandler.getASCIICharset()));
			AlgoParamHandler.readString(currentAlgos, false);
		}
		boolean contentWasSaved = false;
		NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(
				owner, PeaProperties.getVmBridge().translate("change_password"), null,
				owner.getLocation());
		char[] newPsw = newPswDialog.getDialogInput();
		if (newPsw == null) {
			TestLog.v(getClass(), "No password change");
			return null;
		}

		//==============================================
		// new Nonce for every new password
		byte[] nonce = new RandomStuff().createRandomBytes(Attachments.getNonceSize());
		Attachments.setNonce(nonce);
		//==============================================

		try {
			setMessage("... " + PeaProperties.getVmBridge().translate("change_password") + " " +
					PeaProperties.getVmBridge().translate("in_progress")  + "... ", false);
			//messageLabel.setForeground(Color.BLACK);
			//messageLabel.setText("... " + PeaProperties.getVmBridge().translate("change_password") + " " +
			//		PeaProperties.getVmBridge().translate("in_progress")  + "... ");
		} finally {
			// for cloud password
			byte[] previousKey = null;
			BlockCipher previousCipher = null; // cipher might change...
			Digest previousHash = null;// need to check password identifier (nonce is hashed)
			if (owner instanceof PeaLockFrame && ( ! PeaProperties.getFileType().equals("file"))) {
				previousKey = CipherStuff.getSessionKeyCrypt().getKey();
				if (SettingMenu.getAlgoParamsBeforeChange() != null) {
					previousCipher = AlgoParamHandler.getCipherFromString( new String( SettingMenu.getAlgoParamsBeforeChange(), AttachmentHandler.getASCIICharset()));
					previousHash = AlgoParamHandler.getHashFromString(new String( SettingMenu.getAlgoParamsBeforeChange(), AttachmentHandler.getASCIICharset()));
				} else {
					previousCipher = CipherStuff.getCipherAlgo();
					previousHash = HashStuff.getHashAlgo();
				}
			}

			byte[] keyMaterial = PeaControl.deriveKeyFromPsw(newPsw);
			if (keyMaterial == null) {
				new UnexpectedValueException("byte[]", "keyMaterial", "is null").printDescription();
				if (previousKey != null) {
					Zeroizer.zero(previousKey);
				}
				return PeaControl.getDerivationError();				
			}
			// key was derived with new algorithms/password:
			setChangesApplied(true);

			if (newPsw != null){
				Zeroizer.zero(newPsw);
			} 
			if (CipherStuff.getSessionKeyCrypt() == null) {
				new UnexpectedValueException("SessionKeyCrypt", "CipherStuff.getSessionKeyCrypt()", "is null").printDescription();
				return PeaProperties.getVmBridge().translate("unexpected_error") + " (SessionKeyCrypt missing)";
			}
			CipherStuff.getSessionKeyCrypt().clearKeys();
			//CipherStuff.resetSessionKeyCrypt();
			CipherStuff.getSessionKeyCrypt().storeKey(keyMaterial);
			setMessage("", false);
			//PeaProperties.getOsBridge().translate("done"));
			
			// set properties files for new scheme
			if (owner instanceof PeaLockFrame) {
//				String currentAlgos = AlgoParamHandler.writeString(false);
//				PeaProperties.setAlgoParamString(currentAlgos.getBytes(AttachmentHandler.getASCIICharset()));
				//TestLog.v(PeaLockFrame.class,"Current algo: " + new String(AlgoParamHandler.writeString(false)));

				if ( ! PeaProperties.getFileType().equals("file")) {
					String[] currentFileNames = null;
					if (fileNamesToApply != null) {
						currentFileNames = fileNamesToApply;
					} else {
						currentFileNames = ((PeaLockFrame) owner).getCurrentFileNames();
					}
					if (currentFileNames != null) {
						if (PeaProperties.getFileType().equals("image")
								|| PeaProperties.getFileType().equals("calendar")) { // TODO Notebook
							// re-encrypt files: Large files are stored in plaintext on disk to re-encrypt
							// inform the user and ask to accept the risk:
							boolean warn = false;
							long warningLen = 1024*1024*256;// 256 MB
							boolean acceptRisk = false;
							for (String s : currentFileNames) {
								if (s != null) {
									File f = new File(s);
									if (f.exists() && f.isFile() && f.canRead()) {
										if (f.length() > warningLen) {
											warn = true;
											break;
										}
									}
								}
							}
							
							if (warn == true) {
								int warnResult = PeaDialog.showQuestion(owner, 
										PeaProperties.getVmBridge().translate("file_too_large") +"\n"
										+ "Can not re-encrypt in RAM. \n"
												+ "Re-encrypt the files on disk?\n"
										+ "  (Accept the risk that files are shortly unencrypted)", 
										PeaProperties.getVmBridge().translate("warning"), 0);// yes/no
								if (warnResult == 0) {
									acceptRisk = true;
								}
							}
							//save
							//ContentUtilities.changePassword( currentFileNames, this);	// file names to change
							if (previousKey == null || keyMaterial == null) {
								TestLog.ve(PeaLockFrame.class, "Key is null", 5);
								return PeaProperties.getVmBridge().translate("unexpected_error") + " (null key)";
							}
							TestLog.v(PeaLockFrame.class, "Re-encrypt " + currentFileNames.length + " files with new key...");
							String reencryptError = ContentUtilities.reencrypt(currentFileNames, Arrays.copyOf(previousKey, previousKey.length), 
									CipherStuff.getSessionKeyCrypt().getKey(), 
									previousCipher, CipherStuff.getCipherAlgo(), previousHash, HashStuff.getHashAlgo(), acceptRisk);
							if (reencryptError != null) {
								PeaDialog.showMultiLineMessage(owner, reencryptError, 
										PeaProperties.getVmBridge().translate("error"), 0);
							}
							// upload all re-encrypted cloud files:
/*							ArrayList<String> filesToUpload = new ArrayList<String>();
							for (String fileToUpload : currentFileNames) {
								if (fileToUpload.startsWith(CloudControl.getTmpDir())) {
									filesToUpload.add(fileToUpload);
								} else { 
									// local file, skip, already saved
								}
							}
							// save
							String uploadError = ContentUtilities.saveTmpFileToSeveralProviders(Converter.arrayListToArray(filesToUpload));
							if (uploadError != null) {
								PeaDialog.showMessage(JREProperties.getMainWindow(), 
										uploadError, null, 0);
							}*/
						}
						String newSaltAsHex = Converter.bytes2hex(KeyDerivation.getSalt());
						String newAlgoParamAsHex = Converter.bytes2hex(PeaProperties.getAlgoParamString());
						if (newSaltAsHex != null && newAlgoParamAsHex != null) {
							//boolean passwordChanged = false;
							// for these providers the password in properties was changed:
							ArrayList<String> changedProviders = new ArrayList<String>();
							for (String openedFileName : currentFileNames) {
								if (openedFileName.startsWith(CloudControl.getTmpDir())) {
									String providerName = CloudControl.getProviderNameFromTmpFile(openedFileName);
									String fileNameToRemove = CloudControl.getFileNameFromTmpFile(openedFileName);
									if (providerName != null && fileNameToRemove != null) {
										//cloudFileFound = true;
										int previosSubSetIndex = CloudPropertyHandler.removeFileFromAnyProviderSet(providerName, fileNameToRemove, null);
// TODO need removeResult to store psw later
										if (previosSubSetIndex >= 0 ) {//|| algorithmChange == false) { // file was removed
// TODO BUG if only password changed: no file was removed
// -> no cloud password is changed
// TODO store password only once per provider, not for every file
							
											String plainPsw = null;
											if ( ! changedProviders.contains(providerName) ) {//passwordChanged == false) {
												// get password from set, the file was removed from
												String[] subSetRemove = CloudPropertyHandler.getSubSet(providerName, previosSubSetIndex, null);

												if (subSetRemove[2] == null || subSetRemove[2].length() == 0) {
													TestLog.e(PeaLockFrame.class, "Missing password from set, a file was removed from...");
												} else {
													plainPsw = CloudPropertyHandler.decryptPassword(subSetRemove[2], Arrays.copyOf(previousKey, previousKey.length), previousCipher);
												}
											}
											int extToAdd = CloudPropertyHandler.getSubSetExtension(providerName, newSaltAsHex, newAlgoParamAsHex, null);
											String[] fileNamesToAdd = {fileNameToRemove};
											if (extToAdd >= 0) { // exists already												
												String addError = CloudPropertyHandler.addFileNamesToProperties(providerName, fileNamesToAdd, 
														newSaltAsHex, newAlgoParamAsHex);
												if (addError != null) {
													TestLog.e(getClass(), addError);
												}
												// change password:
												if ( ! changedProviders.contains(providerName) ) {//passwordChanged == false) {
													String storeError = CloudPropertyHandler.storeEncryptedPasswordFromProvider(
															CloudPropertyHandler.encryptPassword(plainPsw), providerName, 
															newSaltAsHex, newAlgoParamAsHex, null);
													if (storeError != null) {
														TestLog.e(PeaLockFrame.class, "Could not change cloud password in properties");
													} else {
														TestLog.v(PeaLockFrame.class, providerName +  " - Cloud password in properties changed for " + " " + fileNameToRemove);
														changedProviders.add(providerName);
														//passwordChanged = true;
													}
												}
											} else {
												// create new set, get password (this is done only once, next file will be added)
												int newExt = CloudPropertyHandler.createSubSet(providerName, newSaltAsHex, newAlgoParamAsHex, 
														plainPsw, null);
												
												if (newExt == -1) {//failed
													TestLog.e(PeaLockFrame.class, "Could not create new property set for " + providerName);													
												} else {
													changedProviders.add(providerName);
													//passwordChanged = true;
													// add removed file name to new set
													String addError = CloudPropertyHandler.addFileNamesToProperties(providerName, fileNamesToAdd, newSaltAsHex, newAlgoParamAsHex);
													if (addError != null) {//failed
														TestLog.e(PeaLockFrame.class, "Could not add file " + fileNameToRemove + " to " + providerName);
													}
												}
											}
										} else {
											TestLog.v(PeaLockFrame.class, "File name was not removed from property set: " + fileNameToRemove);
										}
									} else {
										TestLog.ve(PeaLockFrame.class, "provider/file name string is null", 5);
										//return PeaProperties.getVmBridge().translate("unexpected_error") + " (missing provider/file name)";
									}
								}
							}
						} else {
							TestLog.ve(getClass(), "Salt or algorithm-parameter string is null", 5);
							return PeaProperties.getVmBridge().translate("unexpected_error") + " (missing salt/algo-string)";
						}
					}
					PeaControl.setAvailableFileNames(Converter.arrayToArrayList(currentFileNames));
					// set new FileTypePanel, CloudFilePanel...
	// TODO test//FileComposer newFc = new FileComposer(new FileModel(false, PeaProperties.getAlgoParamString(), KeyDerivation.getSalt()));
					FileComposer newFc = new FileComposer(new FileModel(true, PeaProperties.getAlgoParamString(), KeyDerivation.getSalt()));

					FileTypePanel newFtp = new FileTypePanel(100,100, true, false, newFc, this);
					
					
					// set new panel:
					JREProperties.setTypePanel(newFtp);
					CloudFilePanel newCfp = new CloudFilePanel(owner, newFtp, Converter.bytes2hex(KeyDerivation.getSalt()), 
							Converter.bytes2hex(PeaProperties.getAlgoParamString()));
					// set CloudFilePanel
					newFtp.setCloudFilePanel(newCfp);
					if (currentFileNames != null) {
						for (String newFileName : currentFileNames) {
							if (newFileName.contains(CloudControl.getTmpDir())) {
								String providerName = CloudControl.getProviderNameFromTmpFile(newFileName);
								String usedFileName = CloudControl.getFileNameFromTmpFile(newFileName);
								// add to used files
								newCfp.getCloudControl().addUsedFileNameToProvider(providerName, usedFileName);
							}
						}
					} else {
						TestLog.ve(PeaLockFrame.class, "Missing current file names", 5);
					}
					// Now save the content with the new key
					String saveError = saveCurrentContent(); // TODO implement for all PEAs except Notebook and Image (is already encrypted)
					if (saveError != null) {
						TestLog.e(PeaLockFrame.class, "Could not save current file - " + saveError);
						TestLog.ve(PeaLockFrame.class, "Could not save current file - " + saveError, 5);
					} else {
						contentWasSaved = true;
						TestLog.o(PeaLockFrame.class, "Content was saved");
					}
				} // not File Lock PEA
			}
			if (previousKey != null) {
				Zeroizer.zero(previousKey);	
			}
		}
		SettingMenu.setAlgoParamsBeforeChange(null);
		if (contentWasSaved == false) {
			// the content is not re-encrypted:
			markUnsavedContentChanges();
		} else {
			markContentIsSaved();
		}		
		return null;
	}
	
	
	/**
	 * Decrypt the content of a file and load the decrypted
	 * content to the current view.
	 * 
	 * @param fileName	the name of the file to decrypt and load
	 * 
	 * @return an error message in the form "file name - error", or null (success)
	 */
	public abstract String decryptAndLoadFileToView(String fileName);
	
	/**
	 * Choose file names to apply changes (password, algorithms changes).
	 * 
	 * @return	the file names to apply changes to
	 */
	public abstract String[] chooseFileNamesToApply();
	
	/**
	 * Get the associated file names with path of the
	 * currently opened content (Tab in LockFrameEditor (one file + backup files), 
	 * image in LockFrameImage (one file), 
	 * calendar files in LockFrameCalendar (several files))
	 * 
	 * @return	the file names which is associated with 
	 * 			the currently shown content or null if there is no
	 */
	public abstract String[] getCurrentFileNames();
	
	/**
	 * Set the associated file names with path of the
	 * currently opened content (Tab in LockFrameEditor (one file), 
	 * image in LockFrameImage (one file), 
	 * calendar files in LockFrameCalendar (several files))
	 * 
	 * @param fileNamesToSet	the file names to set
	 */
	public abstract void setCurrentFileNames(String[] fileNamesToSet);

	/**
	 * Set a message in the PeaLockFrame. 
	 * This will be shown on the bottom of the frame. 
	 * Errors are shown in color red. 
	 * 
	 * @param string	the message
	 * @param isError	true, if the message is an error
	 */
	public abstract void setMessage(String string, boolean isError);


	/**
	 * Check if the currently open content 
	 * was modified and not saved manually and ask 
	 * to save the content in this case. 
	 * 
	 * @return	true if content is saved if required,
	 * 			false if unsaved content remains or
	 * 			no unsaved changes exist
	 */
	public abstract boolean checkUnsavedContentChanges();
	
	
	/**
	 * Mark that there are content changes that have not been saved yet.
	 */
	public abstract void markUnsavedContentChanges();
	
	/**
	 * Mark that all current content was saved
	 */
	public abstract void markContentIsSaved();
	
	
	/**
	 * Update the PEA specific model if there is one
	 * with files that are uploaded to the cloud
	 * 
	 * @param files			the file that are uploaded to the cloud
	 * @param providerName	the provider name, the files were uploaded to
	 * 
	 */
	public abstract void updateModelWithCloudFiles(File[] files, String providerName);

	
	/**
	 * Store the current content in a temporary file, that
	 * is used as local backup of a cloud file:
	 *   System.getProperty("java.io.tmpdir") / providerName / fileName
	 *   
	 * @param providerName		the name of the cloud provider
	 * @param fileNameToStore	the file name to store without path
	 * 
	 * @return	an error message, if an error occurred or null
	 */
	public abstract String saveCurrentContentInCloudTempFile(String providerName, String fileNameToStore);
	
	/**
	 * Saves the content of all files from getCurrentFileName(). 
	 * This is performed after the password was changed. 
	 * 
	 * @return null for success, an error message otherwise
	 */
	public abstract String saveCurrentContent();


	public boolean isChangesApplied() {
		return changesApplied;
	}


	public void setChangesApplied(boolean changesApplied) {
		this.changesApplied = changesApplied;
	}
	
	/**
	 * Encrypt the list of files with the session key (used in File Lock PEA)
	 * 
	 * @return a list with first element: error message and 
	 *			all following elements: failed files to be deleted
	 */
	//public abstract ArrayList<String>  encryptFiles(ArrayList<File> list);

}
