package cologne.eck.all_peas.gui.menu;


/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */
import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;

import javax.accessibility.AccessibleContext;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.digests.Blake2bDigest;
import org.bouncycastle.crypto.digests.SHA3Digest;
import org.bouncycastle.crypto.engines.AESEngine;
import org.bouncycastle.crypto.engines.ThreefishEngine;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.KeyFileDialogs;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.peafactory.crypto.kdf.Argon2KDF;
import cologne.eck.peafactory.crypto.kdf.CatenaKDF;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.peafactory.gui.kdf_settings.Argon2Setting;
import cologne.eck.peafactory.gui.kdf_settings.CatenaSetting;
import cologne.eck.tools.PropertyHandler;
//import cologne.eck.peafactory.gui.kdf_settings.Argon2Setting;
//import cologne.eck.peafactory.gui.kdf_settings.CatenaSetting;
import cologne.eck.tools.TestLog;



@SuppressWarnings("serial")
public class SettingMenu extends JMenu implements ActionListener {
	
	private ResourceBundle languagesBundle;
	
	private Window owner;// lock frame or password dialog
	
	// the algorithm-parameter string before changes were applied
	private static byte[] algoParamsBeforeChange = null;
	
	private boolean changeWarningShown = false;
	
	/**
	 * Help menu for menu bar
	 * 
	 * @param _owner	the root window (PswDialogView or lockFrame)
	 */
	public SettingMenu(Window _owner) {
		if (_owner == null) {
			TestLog.ve(getClass(), "Missing owner window",5);
		}
		this.owner = _owner;
		Font font = new Font(Font.SANS_SERIF, Font.PLAIN, PeaProperties.getFontSize());
		this.setFont(font);
		languagesBundle = (ResourceBundle) JREProperties.getBundle();
		this.setText(languagesBundle.getString("settings"));
		this.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu for settings.");

		String c = "";
		if (owner instanceof PswDialogView) {//.getName().equals("EncryptedDialog")) {
			c = languagesBundle.getString("default") + " " + languagesBundle.getString("cipher");
		} else { // lockFrame
			c = languagesBundle.getString("cipher");
		}
		JMenu cipherMenu = new JMenu(c);
		cipherMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component displays a list of available ciphers.");
		cipherMenu.setFont(font);
		this.add(cipherMenu);
		
		JMenuItem threefishItem = new JMenuItem("Threefish-512");
		threefishItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects the cipher Threefish-512 for encryption.");
		threefishItem.setActionCommand("threefish512");
		threefishItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		threefishItem.setMnemonic(KeyStroke.getKeyStroke("T").getKeyCode());
		threefishItem.addActionListener(this);
		cipherMenu.add(threefishItem);
		
		JMenuItem aesItem = new JMenuItem("AES-256");
		aesItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects the cipher AES-256 for encryption.");
		aesItem.setActionCommand("aes256");
		aesItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		aesItem.setMnemonic(KeyStroke.getKeyStroke("A").getKeyCode());
		aesItem.addActionListener(this);
		cipherMenu.add(aesItem);
		
		

		String h = "";
		if (owner  instanceof PswDialogView) {//.getName().equals("EncryptedDialog")) {
			h = languagesBundle.getString("default") + " " + languagesBundle.getString("hash");
		} else { // lockFrame
			h = languagesBundle.getString("hash");
		}
		JMenu hashMenu = new JMenu(h);
		hashMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component displays a list of available hash functions.");
		hashMenu.setFont(font);
		this.add(hashMenu);
		
		JMenuItem blake2BItem = new JMenuItem("Blake2b");
		blake2BItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects the hash function Blake2b.");
		blake2BItem.setActionCommand("blake2b");
		blake2BItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		blake2BItem.setMnemonic(KeyStroke.getKeyStroke("B").getKeyCode());
		blake2BItem.addActionListener(this);
		hashMenu.add(blake2BItem);
		
		JMenuItem sha3Item = new JMenuItem("SHA-3");
		sha3Item.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects the hash function SHA-3.");
		sha3Item.setActionCommand("sha3");
		sha3Item.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		sha3Item.setMnemonic(KeyStroke.getKeyStroke("S").getKeyCode());
		sha3Item.addActionListener(this);
		hashMenu.add(sha3Item);
		

		String k = "";
		if (owner  instanceof PswDialogView) {//.getName().equals("EncryptedDialog")) {
			k = languagesBundle.getString("default") + " " + languagesBundle.getString("key_derivation");
		} else { // lockFrame
			k = languagesBundle.getString("key_derivation");
		}
		JMenu kdfMenu = new JMenu(k);
		kdfMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component displays a list of available key derivation functions.");
		kdfMenu.setFont(font);
		this.add(kdfMenu);
		
		JMenuItem argon2Item = new JMenuItem("Argon2id");
		argon2Item.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects the key derivation Argon2id.");
		argon2Item.setActionCommand("argon2id");
		argon2Item.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		argon2Item.setMnemonic(KeyStroke.getKeyStroke("A").getKeyCode());
		argon2Item.addActionListener(this);
		kdfMenu.add(argon2Item);
		
		JMenuItem catenaDragItem = new JMenuItem("Catena-Dragonfly");
		catenaDragItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects the key derivation Catena-Dragonfly.");
		catenaDragItem.setActionCommand("catenaDrag");
		catenaDragItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		catenaDragItem.setMnemonic(KeyStroke.getKeyStroke("D").getKeyCode());
		catenaDragItem.addActionListener(this);
		kdfMenu.add(catenaDragItem);
		
		JMenuItem catenaButtItem = new JMenuItem("Catena-Butterfly");
		catenaButtItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects the key derivation Catena-Butterfly.");
		catenaButtItem.setActionCommand("catenaButt");
		catenaButtItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		catenaButtItem.setMnemonic(KeyStroke.getKeyStroke("B").getKeyCode());
		catenaButtItem.addActionListener(this);
		kdfMenu.add(catenaButtItem);

		
		String p = "";
		if (owner instanceof PswDialogView) {//.getName().equals("EncryptedDialog")) {
			p = languagesBundle.getString("default") + " " + languagesBundle.getString("parameter");
		} else { // lockFrame
			p = languagesBundle.getString("parameter");
		}
		JMenu paramMenu = new JMenu(p);
		paramMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component displays a list of parameter setting dialogs.");
		paramMenu.setFont(font);
		kdfMenu.add(paramMenu);
		
		JMenuItem argon2ParamItem = new JMenuItem("Argon2 - " + languagesBundle.getString("parameter"));
		argon2ParamItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to set Argon2 parameters.");
		argon2ParamItem.setActionCommand("argon2Params");
		argon2ParamItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		argon2ParamItem.setMnemonic(KeyStroke.getKeyStroke("A").getKeyCode());
		argon2ParamItem.addActionListener(this);
		paramMenu.add(argon2ParamItem);
		
		
		JMenuItem catenaParamItem = new JMenuItem("Catena - " + languagesBundle.getString("parameter"));
		catenaParamItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to set Catenay parameters.");
		catenaParamItem.setActionCommand("catenaParams");
		catenaParamItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		catenaParamItem.setMnemonic(KeyStroke.getKeyStroke("C").getKeyCode());
		catenaParamItem.addActionListener(this);
		paramMenu.add(catenaParamItem);		
		
		JMenuItem keyFileItem = new JMenuItem("key file");
		keyFileItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to create a key file.");
		keyFileItem.setActionCommand("keyFile");
		keyFileItem.setFont(font);
		//webItem.setMnemonic(KeyEvent.VK_I);
		keyFileItem.setMnemonic(KeyStroke.getKeyStroke("F").getKeyCode());
		keyFileItem.addActionListener(this);
		this.add(keyFileItem);
	}
	

	
	private void apply() {
		
		//TestLog.o(getClass(), "apply...");
		// set the the previous algorithm-parameter string
		algoParamsBeforeChange = PeaProperties.getAlgoParamString();
		//TestLog.v(SettingMenu.class, "Store previous algoParam: " + new String(PeaProperties.getAlgoParamString()));
		if (owner == null ) {
			TestLog.ve(getClass(), "Missing owner window", 5);
		}
		if (owner != null && owner instanceof PswDialogView) {
			// password was not yet typed
			String defaultAlgos = AlgoParamHandler.writeString(true);
			PeaProperties.setDefaultAlgoParamString(defaultAlgos);
			TestLog.o(getClass(), "New default algorithms and parameters: " + AlgoParamHandler.getReadableString(defaultAlgos, false));
			new PropertyHandler().setOneProperty("default_algo_param", defaultAlgos);

		} else {
			if (owner instanceof PeaLockFrame) {
				// do not set PeaProperties, this is done later
				// ask to apply changes or make further settings
				// TODO MultiFileSelection
				String[] fileNamesToApply =((PeaLockFrame) owner).chooseFileNamesToApply();
				String errorMessage = ((PeaLockFrame) owner).changePasswordOrApplyChanges(owner, fileNamesToApply, true);
				if (errorMessage != null) {
					PeaDialog.showMessage(owner, 
							errorMessage, 
							PeaProperties.getVmBridge().translate("error"), 0);
				}
			} else {
				TestLog.ve(getClass(), "Invalid class of owner", 5);
			}
		}
	}
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		
		if (CloudControl.getNumberOfRunningCloudProcesses() > 0) {
			// TODO only cancel if same file is to be uploaded twice
			// register uploaded files in CloudControl
			PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("cloud_process_warning"));
			return;
		}
		
		if ( changeWarningShown == false &&
				(PeaProperties.getFileType().equals("text file"))// || PeaProperties.getFileType().equals("image")) 
				&&  owner instanceof PeaLockFrame 
				&& PeaControl.getAvailableFileNames().size() > 1) {
			boolean cloudFileIncluded = false;
			// show warning: other files cannot be opened anymore:
			String[] openFiles = ((PeaLockFrame) owner).getCurrentFileNames();
			// for image and editor, this is always one file
			if (openFiles[0].startsWith(CloudControl.getTmpDir())) {
				cloudFileIncluded = true;
			}
			// confirm warning: other tab cannot be opened
			String[] options = {PeaProperties.getVmBridge().translate("cancel"), 
					PeaProperties.getVmBridge().translate("ok")};
			String cloudWarning = "";
			if ( cloudFileIncluded == true) {
				cloudWarning = "\n" + PeaProperties.getVmBridge().translate("cloud_access_warning");
			}
			int opt = PeaDialog.showOptions(this, 
					PeaProperties.getVmBridge().translate("change_apply_this_tab") + cloudWarning, 
					PeaProperties.getVmBridge().translate("warning"), 0, 2, options, 0);
			if (opt != 1) {
				return;
			}
		}
		changeWarningShown = true;
		
		boolean setDefaults = false;
		if (owner instanceof PswDialogView) {
			setDefaults = true;
		}
		String command = ae.getActionCommand();

		if (command.equals("threefish512")){
			BlockCipher three512 = new ThreefishEngine(512);
			if (setDefaults == true) {
				if (PeaProperties.getDefaultCipher() instanceof ThreefishEngine) {
					TestLog.v(getClass(), "Same Algo chosen...");
					PeaDialog.showMessage(owner, 
							PeaProperties.getVmBridge().translate("no_change"), 
							null, -1);
					return;
				}
				PeaProperties.setDefaultCipher(three512);	
			} else {			
				if (CipherStuff.getCipherAlgo() instanceof ThreefishEngine) {
					TestLog.v(getClass(), "Same Algo chosen...");
					PeaDialog.showMessage(owner, 
							PeaProperties.getVmBridge().translate("no_change"), 
							null, -1);
					return;
				}
				CipherStuff.setCipherAlgo(three512);					
			}
			apply();

		} else if (command.equals("aes256")){
			
			AESEngine aes = new AESEngine();
			if (setDefaults == true) {
				if (PeaProperties.getDefaultCipher() instanceof AESEngine) {
					TestLog.v(getClass(), "Same Algo chosen...");
					PeaDialog.showMessage(owner, 
							PeaProperties.getVmBridge().translate("no_change"), 
							null, -1);
					return;
				}
				PeaProperties.setDefaultCipher(aes);
			} else {				
				if (CipherStuff.getCipherAlgo() instanceof AESEngine) {
					TestLog.v(getClass(), "Same Algo chosen...");
					PeaDialog.showMessage(owner, 
							PeaProperties.getVmBridge().translate("no_change"), 
							null, -1);
					return;
				}
				CipherStuff.setCipherAlgo(aes);
			}
			apply();
			
		} else if (command.equals("blake2b")) {
			
			Blake2bDigest blake2b = new Blake2bDigest();
			if (setDefaults == true) {
				if (PeaProperties.getDefaultDigest() instanceof Blake2bDigest) {
					TestLog.v(getClass(), "Same Algo chosen...");
					PeaDialog.showMessage(owner, 
							PeaProperties.getVmBridge().translate("no_change"), 
							null, -1);
					return;
				} 
				PeaProperties.setDefaultDigest(blake2b);
			} else {
				if (HashStuff.getHashAlgo() instanceof Blake2bDigest) {
					TestLog.v(getClass(), "Same Algo chosen...");
					PeaDialog.showMessage(owner, 
							PeaProperties.getVmBridge().translate("no_change"), 
							null, -1);
					return;
				}
				HashStuff.setHashAlgo(blake2b);
				//HashStuff.setHMacAlgo(blake2b);
			}
			apply();
			
		} else if (command.equals("sha3")) {
			
			SHA3Digest sha3 = new SHA3Digest();
			if (setDefaults == true) {
				if (PeaProperties.getDefaultDigest() instanceof SHA3Digest) {
					TestLog.v(getClass(), "Same Algo chosen...");
					PeaDialog.showMessage(owner, 
							PeaProperties.getVmBridge().translate("no_change"), 
							null, -1);
					return;
				} 
				PeaProperties.setDefaultDigest(sha3);
			} else {
				if (HashStuff.getHashAlgo() instanceof SHA3Digest) {
					TestLog.v(getClass(), "Same Algo chosen...");
					PeaDialog.showMessage(owner, 
							PeaProperties.getVmBridge().translate("no_change"), 
							null, -1);
					return;
				}
				HashStuff.setHashAlgo(sha3);
				//HashStuff.setHMacAlgo(sha3);
			}
			
			TestLog.v(SettingMenu.class, "Set " + HashStuff.getHashAlgo().getAlgorithmName()
					+ " / " + PeaProperties.getDefaultDigest().getAlgorithmName());
			apply();

		} else if (command.equals("argon2id")){

			Argon2KDF argon2 = new Argon2KDF();
			if (setDefaults == true) {
				PeaProperties.setDefaultKDF(argon2);
				PeaProperties.setDefaultKDFInstanceAppendix("id");
				PeaProperties.setDefaultTCost(Argon2KDF.getIterations());
				PeaProperties.setDefaultMCost(Argon2KDF.getMemoryCost());
				PeaProperties.setDefaultParallel(Argon2KDF.getLanes());
			} else {
				KeyDerivation.setKdf(argon2);
			}
			apply();

		} else if (command.equals("catenaDrag")){
			CatenaKDF catD = new CatenaKDF();
			if (setDefaults == true) {
				PeaProperties.setDefaultKDF(catD);
				PeaProperties.setDefaultKDFInstanceAppendix("Dragonfly-Full");
				PeaProperties.setDefaultTCost(CatenaKDF.getLambdaDragonfly());
				PeaProperties.setDefaultMCost(CatenaKDF.getGarlicDragonfly());
				PeaProperties.setDefaultParallel(0);
			} else {
				CatenaKDF.setVersionID("Dragonfly-Full");
				KeyDerivation.setKdf(catD);
			}
			apply();

		} else if (command.equals("catenaButt")) {

			CatenaKDF catB = new CatenaKDF();
			if (setDefaults == true) {
				PeaProperties.setDefaultKDFInstanceAppendix("Butterfly-Full");
				PeaProperties.setDefaultKDF(catB);
				PeaProperties.setDefaultTCost(CatenaKDF.getLambdaButterfly());
				PeaProperties.setDefaultMCost(CatenaKDF.getGarlicButterfly());
				PeaProperties.setDefaultParallel(0);
			} else {
				CatenaKDF.setVersionID("Butterfly-Full");
				KeyDerivation.setKdf(catB);
			}
			apply();
			
		} else if (command.equals("argon2Params")) {
			
			@SuppressWarnings("unused")
			Argon2Setting argon2Setting = new Argon2Setting(SwingUtilities.getWindowAncestor(this));
			// do not apply: This is done in dialog when setting is completed

		} else if (command.equals("catenaParams")) {

			@SuppressWarnings("unused")
			CatenaSetting catenaSetting = new CatenaSetting(SwingUtilities.getWindowAncestor(this));
			// do not apply: This is done in dialog when setting is completed


		} else if (command.equals("keyFile")) {

			boolean defaultOnly = false;

			if (owner != null && owner instanceof PswDialogView) { // only default key file allowed
				if (PeaProperties.getDefaultKeyFileName() != null) {
					int applyKeyFile = PeaDialog.showQuestion(SwingUtilities.getWindowAncestor(this), 
							PeaProperties.getVmBridge().translate("default_key_file_selected") + ": \n"
							+ PeaProperties.getDefaultKeyFileName() + "\n\n"
							+ PeaProperties.getVmBridge().translate("use_this_file"),  
							null, 0);
					if (applyKeyFile == 0) { // yes
						//createKeyFile = false;

						// apply key file to default files
						PeaProperties.setDefaultKeyFileFlag(1); // default 
						apply();
						return;
					} else if (applyKeyFile == 1) { // No
						setNonDefaultKeyFile(true);
						return;
						
					} else {
						return;
					}
				}
				defaultOnly = true;

			} else {  // PeaLockFrame:

				if (PeaProperties.getDefaultKeyFileName() != null) {
					int applyKeyFile = PeaDialog.showQuestion(SwingUtilities.getWindowAncestor(this), 
							PeaProperties.getVmBridge().translate("default_key_file_selected") + ": \n"
									+ PeaProperties.getDefaultKeyFileName() + "\n\n"
									+ PeaProperties.getVmBridge().translate("use_this_file"), 
							null, 0);
					if (applyKeyFile == 0) { // yes
						//createKeyFile = false;

						// apply key file to current files
						PeaProperties.setKeyFileFlag(1); // default 

						apply();
						return;
					} else if (applyKeyFile == -1 || applyKeyFile == 2) { // close or cancel
						return;
					} else if (applyKeyFile == 1) { // No
						setNonDefaultKeyFile(false);
						return;
					}
				}
			}

			TestLog.o(getClass(), "Create key file...");
			int keyFileType  = KeyFileDialogs.setKeyFileStatus(defaultOnly);
			if (keyFileType == -1) {
				//PeaDialog.showMessage(SwingUtilities.getWindowAncestor(this), PeaProperties.getVmBridge().translate("error"), 
				//		PeaProperties.getVmBridge().translate("error"), 0);
				return;
			} else {	
				TestLog.o(getClass(), PeaProperties.getVmBridge().translate("key_file") + ": " +
						PeaProperties.getVmBridge().translate("success"));
				apply();
				return;
			}


		} else {
			TestLog.o(getClass(), "Invalid command in help menu");
			return;
		}

	}
	/**
	 * Set a key file status except a new default key file.
	 * 
	 * @param defaultProperties	true: set default key file flag
	 */
	private void setNonDefaultKeyFile(boolean defaultProperties) {
		// choose a key file
		String[] options = { 			
				PeaProperties.getVmBridge().translate("no_key_file"),
				PeaProperties.getVmBridge().translate("gen_key_file_manually_opened"), 
				PeaProperties.getVmBridge().translate("choose_key_file_automatically_opened") };
		int keyFileType = PeaDialog.showOptionYAxis(JREProperties.getMainWindow(), 
				PeaProperties.getVmBridge().translate("choose_key_file_kind")+ "\n",
				//	+ PeaProperties.getVmBridge().translate("default_key_file_info"), 
				PeaProperties.getVmBridge().translate("key_file"), 3, options, 0);
		TestLog.v(SettingMenu.class, "Option: " + keyFileType);		
		File keyFile = null;
		if (keyFileType == 0) { // no key file
			if (defaultProperties == true) {
				PeaProperties.setDefaultKeyFileFlag(0);
			} else {
				PeaProperties.setKeyFileFlag(0);
			}
			apply();
			return;

		} else if (keyFileType == 2) { // manually choose

			PeaFileChooser pfc = new PeaFileChooser(JREProperties.getMainWindow());
			pfc.customizeFileChooser(PeaProperties.getVmBridge().translate("key_file"), 
					PeaProperties.getVmBridge().translate("choose_key_file"), 
					true);
			pfc.setFileSelectionMode(PeaFileChooser.FILES_ONLY);
			//pfc.setSelectedFile(new File("data.txt"));
			pfc.showOpenDialog(JREProperties.getMainWindow());
			keyFile = pfc.getSelectedFile();	
			if (keyFile == null) {			
				return;
			}
		} else if (keyFileType == 1) { // create
			keyFile = KeyFileDialogs.generateKeyFile();
			if (keyFile == null) {
				return;
			} 
		} else {
			return;
		}
		if (defaultProperties == true) {
			PeaProperties.setDefaultKeyFileFlag(2);
		} else {
			PeaProperties.setKeyFileFlag(2);
		}
		PeaDialog.showMessage(JREProperties.getMainWindow(), 					
				PeaProperties.getVmBridge().translate("select_file_when_open_files") + ": \n\n" 
						+ keyFile.getAbsolutePath()  + "\n\n"
						+ PeaProperties.getVmBridge().translate("never_modify_or_delete") ); 
		apply();
		return;

	}
	
	/**
	 * Get the algorithm-parameter string before the last changes were applied. 
	 * 
	 * @return	the last algorithm-parameter string as byte array
	 */
    public static byte[] getAlgoParamsBeforeChange() {
		return algoParamsBeforeChange;
	}
    
    public static void setAlgoParamsBeforeChange(byte[] previousAlgoParams) {
    	algoParamsBeforeChange = previousAlgoParams;
    }

	public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleSettingMenu();
        }
        return accessibleContext;
    }



	private class AccessibleSettingMenu extends AccessibleJMenu {
        //Inherit everything, override nothing.
    }
}
