package cologne.eck.image_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2021  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;

import javax.imageio.ImageIO;

import cologne.eck.tools.TestLog;

public class ImageHelper {
	
	
	/**
	 * Handle the opacity of an image
	 * 
	 * @param sourceImage	the image with opacity
	 * 
	 * @return	the image without opacity
	 */
	private static BufferedImage makeOpaque(BufferedImage sourceImage) {

	    int imageWidth = sourceImage.getWidth();
	    int imageHeight = sourceImage.getHeight();
	    int[] pixels = new int[imageWidth * imageHeight];
	    sourceImage.getRGB(0, 0, imageWidth, imageHeight, pixels, 0, imageWidth);
	    BufferedImage newImage = new BufferedImage(imageWidth, imageHeight, BufferedImage.TYPE_INT_RGB);
	    newImage.setRGB(0, 0, imageWidth, imageHeight, pixels, 0, imageWidth);
	    return newImage;
	}
	
	/**
	 * Convert an image into an user chosen format
	 * 
	 * @param sourceImage	the image to convert
	 * @param format	the format as extension string to convert to: jpg, png, gif, bmp
	 * 
	 * @return	the image in user chosen format as array of bytes at index 0,
	 * 			and the format extension as bytes at index 1
	 */
	protected final static byte[] convertImage(BufferedImage sourceImage, String format) {
		
		// Handle opacity: 
		BufferedImage newSourceImage = null;
		if (format.equals("jpg") || format.equals("jpeg")
				|| format.equals("bmp") ) {		
			if (sourceImage.getTransparency() != BufferedImage.OPAQUE) {
				newSourceImage = makeOpaque(sourceImage);
			} else {
				newSourceImage = sourceImage;
			}
		} else {
			newSourceImage = sourceImage;
		}
		
		// Get bytes in new format from source image: 
		ByteArrayOutputStream outBytes = new ByteArrayOutputStream();		
		try {
			boolean result = ImageIO.write(newSourceImage, format, outBytes);
			if (result == false) {
				TestLog.e(ImageHelper.class, "Could not write image bytes...");
				return null;
			}
		} catch (IOException e) {
			TestLog.e(ImageHelper.class, "Could not write image bytes..." + e.getLocalizedMessage(), 5);
			return null;
		}
		
		// Get new image from bytes
		return outBytes.toByteArray();
	}
	
	/**
	 * The method resizes an image to keep the ratio of width and height
	 * to avoid distortions of the image. The result width is never smaller
	 * than the default width / 4. 
	 * 
	 * @param image		the image to resize
	 * @param width		the width of the frame to display the image
	 * @param height	the height of the frame to display the image
	 * 
	 * @return			resulting width ad height, stored in an array:
	 * 					result[0] is the new width, result[1] the new height to display
	 */
	protected final static int[] resizeImage(BufferedImage image, int width, int height){
		// keep ratio of image width and height: resize the image 

		int[] result = new int[2];
		int displayedWidth = width;
		int displayedHeight = height;			
		int imageWidth = image.getWidth();
		int imageHeight = image.getHeight();
		double op = 0; // factor to resize		
		//TestLog.o(getClass(), "image w: " + image.getWidth() + ", h: " + image.getHeight()
		//		+ "\n" + "default w: " + displayedWidth + ", h: " + displayedHeight);
		 
		 if (imageWidth < width) {
			 		
			 op = (double)width / imageWidth;

			 if (imageHeight * op > height) {
				 
				 op = (double)height / imageHeight;
				 // do not resize smaller than half of default width: 
				 // this is normally a rendered PDF
				 if ( (imageWidth * op) < (width / 4) ) {
					 op = 1;//imageWidth / (double)width;
				 }
			 }
			 
			 displayedWidth = (int) (imageWidth * op);
			 displayedHeight = (int) (imageHeight * op);
		 } else {
			 op = imageWidth / (double)width;

			 if ( (imageHeight / op) > height) {
				 op = imageHeight / (double)height;
				 // do not resize smaller than half of default width: 
				 // this is normally a rendered PDF
				 if ( (imageWidth / op) < (width / 4) ) {
					 op = 1;//imageWidth / (double)width;
				 }
			 }
			 
			 displayedWidth = (int) (imageWidth / op);
			 displayedHeight = (int) (imageHeight / op);
		 }	
		 result[0] = displayedWidth;
		 result[1] = displayedHeight;
		 //TestLog.o(getClass(), "result w: " + result[0] + ", h: " + result[1]);
		 return result;
	}

	/**
	 * Checks if a file is a valid image that can be shown
	 * by ImagePEA. The file must exist and must have read access,
	 * otherwise the check fails. The file is at least checked by 
	 * ImageIO, but before - depending on parameters - checked by
	 * it's MIME type ("image") and it's extension (PNG, GIF, BMP, JEPG, JPG) 
	 * 
	 * @param file				the file to check
	 * @param checkExtension	whether to check the extension of the file or not
	 * @param checkMimeType		whether to check the MIME type of the file or not
	 * @param checkContent		check if content of file is image
	 * 
	 * @return	true if the file is a image that can be shown by the Image PEA
	 */
	static boolean checkImage(File file, boolean checkExtension, boolean checkMimeType, boolean checkContent) {
		
		if ( ! file.isFile()) {
			return false;
		}
		
		String fileName = file.getAbsolutePath();
		
		// check extension: 
		if (checkExtension == true) {
			String extension = fileName.substring(fileName.lastIndexOf('.') + 1,
				fileName.length()).toUpperCase();
			//TestLog.o(getClass(), "extension: " + extension);
			if ((extension.equals("PNG") || extension.equals("GIF")
					|| extension.equals("BMP") || extension.equals("JPG")
					|| extension.equals("JPEG")) ){ 
			} else {
				return false;
			}
		}
		// check MIME type:
		if (checkMimeType == true) {
			String mimetype;
			try {
				mimetype = Files.probeContentType(file.toPath());
				//		 String mimetype= new MimetypesFileTypeMap().getContentType(file);
				if (mimetype != null) {
					String type = mimetype.split("/")[0];
					if( ! type.toLowerCase().equals("image")) {
						return false;
					}
				}
			} catch (IOException e) {
				TestLog.e(LockFrameImage.class, "Cant get mime type of " + file.getAbsolutePath());
			}
		}
		
		if (checkContent == true) {
			// check by ImageIO:
			try{
				BufferedImage bi = ImageIO.read(file);	
				if (bi == null) {
					return false;
				}
			} catch (Exception e) {
				return false;
			}
		}
		return true;
	}

}
